@@ -40,6 +40,10 @@
 #include <errno.h>
 #include <stdarg.h>
 
+#ifndef __GNUC__
+#define __inline__ inline
+#endif
+
 #include <alsa/asoundef.h>
 #include <alsa/version.h>
 #include <alsa/global.h>
