@@ -103,7 +103,7 @@ int snd_ctl_close(snd_ctl_t *ctl)
 /**
  * \brief set nonblock mode
  * \param ctl CTL handle
- * \param nonblock 0 = block, 1 = nonblock mode
+ * \param nonblock 0 = block, 1 = nonblock mode, 2 = abort
  * \return 0 on success otherwise a negative error code
  */
 int snd_ctl_nonblock(snd_ctl_t *ctl, int nonblock)
@@ -417,8 +417,10 @@ int snd_ctl_elem_add_enumerated(snd_ctl_t *ctl, const snd_ctl_elem_id_t *id,
 	info->value.enumerated.names_ptr = (uintptr_t)buf;
 	info->value.enumerated.names_length = bytes;
 	p = buf;
-	for (i = 0; i < items; ++i)
-		p = stpcpy(p, names[i]) + 1;
+	for (i = 0; i < items; ++i) {
+		strcpy(p, names[i]);
+		p += strlen(names[i]) + 1;
+	}
 
 	err = ctl->ops->element_add(ctl, info);
 
@@ -2321,8 +2323,8 @@ void snd_ctl_elem_value_copy(snd_ctl_elem_value_t *dst, const snd_ctl_elem_value
 
 /**
  * \brief compare one #snd_ctl_elem_value_t to another
- * \param dst pointer to destination
- * \param src pointer to source
+ * \param left pointer to first value
+ * \param right pointer to second value
  * \return 0 on match, less than or greater than otherwise, see memcmp
  */
 int snd_ctl_elem_value_compare(snd_ctl_elem_value_t *left, const snd_ctl_elem_value_t *right)
