@@ -1,6 +1,6 @@
 /*
  *  HwDep Symbols
- *  Copyright (c) 2001 by Jaroslav Kysela <perex@suse.cz>
+ *  Copyright (c) 2001 by Jaroslav Kysela <perex@perex.cz>
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser General Public License as
@@ -28,7 +28,7 @@ static const char **snd_hwdep_open_objects[] = {
 	
 void *snd_hwdep_open_symbols(void)
 {
-	return (void *)snd_hwdep_open_objects[0];
+	return snd_hwdep_open_objects;
 }
 
 #endif
