@@ -1,7 +1,7 @@
 /**
  * \file mixer/mixer.c
  * \brief Mixer Interface
- * \author Jaroslav Kysela <perex@suse.cz>
+ * \author Jaroslav Kysela <perex@perex.cz>
  * \author Abramo Bagnara <abramo@alsa-project.org>
  * \date 2001
  *
@@ -10,7 +10,7 @@
  */
 /*
  *  Mixer Interface - main file
- *  Copyright (c) 1998/1999/2000 by Jaroslav Kysela <perex@suse.cz>
+ *  Copyright (c) 1998/1999/2000 by Jaroslav Kysela <perex@perex.cz>
  *  Copyright (c) 2001 by Abramo Bagnara <abramo@alsa-project.org>
  *
  *
@@ -743,7 +743,7 @@ int snd_mixer_poll_descriptors_revents(snd_mixer_t *mixer, struct pollfd *pfds,
 	if (nfds == 0)
 		return -EINVAL;
 	res = 0;
-	for (idx = 0; idx < nfds; idx++)
+	for (idx = 0; idx < nfds; idx++, pfds++)
 		res |= pfds->revents & (POLLIN|POLLERR|POLLNVAL);
 	*revents = res;
 	return 0;
@@ -1062,7 +1062,7 @@ int snd_mixer_class_set_private(snd_mixer_class_t *obj, void *private_data)
  * \param private_free Mixer class private data free callback
  * \return zero if success, otherwise a negative error code
  */
-int snd_mixer_class_set_private_free(snd_mixer_class_t *obj, void (*private_free)(snd_mixer_class_t *class))
+int snd_mixer_class_set_private_free(snd_mixer_class_t *obj, void (*private_free)(snd_mixer_class_t *))
 {
 	assert(obj);
 	obj->private_free = private_free;
