@@ -28,4 +28,14 @@
 	snd1_mixer_simple_basic_register
 
 int snd_mixer_simple_none_register(snd_mixer_t *mixer, struct snd_mixer_selem_regopt *options, snd_mixer_class_t **classp);
+
+#ifdef HAVE_LIBDL
 int snd_mixer_simple_basic_register(snd_mixer_t *mixer, struct snd_mixer_selem_regopt *options, snd_mixer_class_t **classp);
+#else
+static inline int snd_mixer_simple_basic_register(snd_mixer_t *mixer,
+						  struct snd_mixer_selem_regopt *options,
+						  snd_mixer_class_t **classp)
+{
+	return -ENXIO;
+}
+#endif
