@@ -103,7 +103,7 @@ int snd_mixer_selem_register(snd_mixer_t *mixer,
 }
 
 #define CHECK_ENUM(xelem) \
-	if (!((sm_selem_t *)(elem)->private_data)->caps & (SM_CAP_PENUM|SM_CAP_CENUM)) \
+	if (!(((sm_selem_t *)(elem)->private_data)->caps & (SM_CAP_PENUM|SM_CAP_CENUM))) \
 		return -EINVAL;
 
 #define COND_CAPS(xelem, what) \
@@ -217,7 +217,7 @@ int snd_mixer_selem_has_common_switch(snd_mixer_elem_t *elem)
  */
 const char *snd_mixer_selem_channel_name(snd_mixer_selem_channel_id_t channel)
 {
-	static const char *array[SND_MIXER_SCHN_LAST + 1] = {
+	static const char *const array[SND_MIXER_SCHN_LAST + 1] = {
 		[SND_MIXER_SCHN_FRONT_LEFT] = "Front Left",
 		[SND_MIXER_SCHN_FRONT_RIGHT] = "Front Right",
 		[SND_MIXER_SCHN_REAR_LEFT] = "Rear Left",
@@ -878,7 +878,6 @@ int snd_mixer_selem_set_capture_switch_all(snd_mixer_elem_t *elem, int value)
 int snd_mixer_selem_is_enumerated(snd_mixer_elem_t *elem)
 {
 	CHECK_BASIC(elem);
-	CHECK_ENUM(elem);
 	return sm_selem_ops(elem)->is(elem, SM_PLAY, SM_OPS_IS_ENUMERATED, 0);
 }
 
@@ -890,7 +889,6 @@ int snd_mixer_selem_is_enumerated(snd_mixer_elem_t *elem)
 int snd_mixer_selem_is_enum_playback(snd_mixer_elem_t *elem)
 {
 	CHECK_BASIC(elem);
-	CHECK_ENUM(elem);
 	return sm_selem_ops(elem)->is(elem, SM_PLAY, SM_OPS_IS_ENUMERATED, 1);
 }
 
@@ -902,7 +900,6 @@ int snd_mixer_selem_is_enum_playback(snd_mixer_elem_t *elem)
 int snd_mixer_selem_is_enum_capture(snd_mixer_elem_t *elem)
 {
 	CHECK_BASIC(elem);
-	CHECK_ENUM(elem);
 	return sm_selem_ops(elem)->is(elem, SM_CAPT, SM_OPS_IS_ENUMERATED, 1);
 }
 
