@@ -152,7 +152,7 @@ static void snd_pcm_copy_dump(snd_pcm_t *pcm, snd_output_t *out)
 	snd_pcm_dump(copy->plug.gen.slave, out);
 }
 
-static snd_pcm_ops_t snd_pcm_copy_ops = {
+static const snd_pcm_ops_t snd_pcm_copy_ops = {
 	.close = snd_pcm_generic_close,
 	.info = snd_pcm_generic_info,
 	.hw_refine = snd_pcm_copy_hw_refine,
@@ -165,6 +165,9 @@ static snd_pcm_ops_t snd_pcm_copy_ops = {
 	.async = snd_pcm_generic_async,
 	.mmap = snd_pcm_generic_mmap,
 	.munmap = snd_pcm_generic_munmap,
+	.query_chmaps = snd_pcm_generic_query_chmaps,
+	.get_chmap = snd_pcm_generic_get_chmap,
+	.set_chmap = snd_pcm_generic_set_chmap,
 };
 
 /**
