@@ -46,7 +46,7 @@ const char *_snd_module_pcm_meter = "";
 struct _snd_pcm_scope {
 	int enabled;
 	char *name;
-	snd_pcm_scope_ops_t *ops;
+	const snd_pcm_scope_ops_t *ops;
 	void *private_data;
 	struct list_head list;
 };
@@ -419,7 +419,7 @@ static int snd_pcm_meter_hw_refine_slave(snd_pcm_t *pcm, snd_pcm_hw_params_t *pa
 static int snd_pcm_meter_hw_params_slave(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 {
 	snd_pcm_meter_t *meter = pcm->private_data;
-	return _snd_pcm_hw_params(meter->gen.slave, params);
+	return _snd_pcm_hw_params_internal(meter->gen.slave, params);
 }
 
 static int snd_pcm_meter_hw_refine(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
@@ -501,7 +501,7 @@ static void snd_pcm_meter_dump(snd_pcm_t *pcm, snd_output_t *out)
 	snd_pcm_dump(meter->gen.slave, out);
 }
 
-static snd_pcm_ops_t snd_pcm_meter_ops = {
+static const snd_pcm_ops_t snd_pcm_meter_ops = {
 	.close = snd_pcm_meter_close,
 	.info = snd_pcm_generic_info,
 	.hw_refine = snd_pcm_meter_hw_refine,
@@ -514,9 +514,12 @@ static snd_pcm_ops_t snd_pcm_meter_ops = {
 	.async = snd_pcm_generic_async,
 	.mmap = snd_pcm_generic_mmap,
 	.munmap = snd_pcm_generic_munmap,
+	.query_chmaps = snd_pcm_generic_query_chmaps,
+	.get_chmap = snd_pcm_generic_get_chmap,
+	.set_chmap = snd_pcm_generic_set_chmap,
 };
 
-static snd_pcm_fast_ops_t snd_pcm_meter_fast_ops = {
+static const snd_pcm_fast_ops_t snd_pcm_meter_fast_ops = {
 	.status = snd_pcm_generic_status,
 	.state = snd_pcm_generic_state,
 	.hwsync = snd_pcm_generic_hwsync,
@@ -542,6 +545,7 @@ static snd_pcm_fast_ops_t snd_pcm_meter_fast_ops = {
 	.poll_descriptors_count = snd_pcm_generic_poll_descriptors_count,
 	.poll_descriptors = snd_pcm_generic_poll_descriptors,
 	.poll_revents = snd_pcm_generic_poll_revents,
+	.may_wait_for_avail_min = snd_pcm_generic_may_wait_for_avail_min,
 };
 
 /**
@@ -603,7 +607,7 @@ static int snd_pcm_meter_add_scope_conf(snd_pcm_t *pcm, const char *name,
 	snd_config_iterator_t i, next;
 	const char *id;
 	const char *lib = NULL, *open_name = NULL, *str = NULL;
-	snd_config_t *c, *type_conf;
+	snd_config_t *c, *type_conf = NULL;
 	int (*open_func)(snd_pcm_t *, const char *,
 			 snd_config_t *, snd_config_t *) = NULL;
 	snd_pcm_meter_t *meter = pcm->private_data;
@@ -960,7 +964,7 @@ const char *snd_pcm_scope_get_name(snd_pcm_scope_t *scope)
  * \param scope PCM meter scope
  * \param val callbacks
  */
-void snd_pcm_scope_set_ops(snd_pcm_scope_t *scope, snd_pcm_scope_ops_t *val)
+void snd_pcm_scope_set_ops(snd_pcm_scope_t *scope, const snd_pcm_scope_ops_t *val)
 {
 	scope->ops = val;
 }
@@ -1143,7 +1147,7 @@ static void s16_reset(snd_pcm_scope_t *scope)
 	s16->old = meter->now;
 }
 
-snd_pcm_scope_ops_t s16_ops = {
+static const snd_pcm_scope_ops_t s16_ops = {
 	.enable = s16_enable,
 	.disable = s16_disable,
 	.close = s16_close,
