@@ -406,7 +406,7 @@ static void snd_pcm_mulaw_dump(snd_pcm_t *pcm, snd_output_t *out)
 	snd_pcm_dump(mulaw->plug.gen.slave, out);
 }
 
-static snd_pcm_ops_t snd_pcm_mulaw_ops = {
+static const snd_pcm_ops_t snd_pcm_mulaw_ops = {
 	.close = snd_pcm_generic_close,
 	.info = snd_pcm_generic_info,
 	.hw_refine = snd_pcm_mulaw_hw_refine,
@@ -419,6 +419,8 @@ static snd_pcm_ops_t snd_pcm_mulaw_ops = {
 	.async = snd_pcm_generic_async,
 	.mmap = snd_pcm_generic_mmap,
 	.munmap = snd_pcm_generic_munmap,
+	.get_chmap = snd_pcm_generic_get_chmap,
+	.set_chmap = snd_pcm_generic_set_chmap,
 };
 
 /**
