@@ -61,6 +61,7 @@ struct _snd_pcm_rate {
 	snd_pcm_channel_area_t *pareas;	/* areas for splitted period (rate pcm) */
 	snd_pcm_channel_area_t *sareas;	/* areas for splitted period (slave pcm) */
 	snd_pcm_rate_info_t info;
+	void *open_func;
 	void *obj;
 	snd_pcm_rate_ops_t ops;
 	unsigned int get_idx;
@@ -69,12 +70,17 @@ struct _snd_pcm_rate {
 	int16_t *dst_buf;
 	int start_pending; /* start is triggered but not commited to slave */
 	snd_htimestamp_t trigger_tstamp;
+	unsigned int plugin_version;
+	unsigned int rate_min, rate_max;
 };
 
+#define SND_PCM_RATE_PLUGIN_VERSION_OLD	0x010001	/* old rate plugin */
+
 #endif /* DOC_HIDDEN */
 
 static int snd_pcm_rate_hw_refine_cprepare(snd_pcm_t *pcm ATTRIBUTE_UNUSED, snd_pcm_hw_params_t *params)
 {
+	snd_pcm_rate_t *rate = pcm->private_data;
 	int err;
 	snd_pcm_access_mask_t access_mask = { SND_PCM_ACCBIT_SHM };
 	snd_pcm_format_mask_t format_mask = { SND_PCM_FMTBIT_LINEAR };
@@ -89,14 +95,18 @@ static int snd_pcm_rate_hw_refine_cprepare(snd_pcm_t *pcm ATTRIBUTE_UNUSED, snd_
 	err = _snd_pcm_hw_params_set_subformat(params, SND_PCM_SUBFORMAT_STD);
 	if (err < 0)
 		return err;
-	err = _snd_pcm_hw_param_set_min(params,
-					SND_PCM_HW_PARAM_RATE, SND_PCM_PLUGIN_RATE_MIN, 0);
-	if (err < 0)
-		return err;
-	err = _snd_pcm_hw_param_set_max(params,
-					SND_PCM_HW_PARAM_RATE, SND_PCM_PLUGIN_RATE_MAX, 0);
-	if (err < 0)
-		return err;
+	if (rate->rate_min) {
+		err = _snd_pcm_hw_param_set_min(params, SND_PCM_HW_PARAM_RATE,
+						rate->rate_min, 0);
+		if (err < 0)
+			return err;
+	}
+	if (rate->rate_max) {
+		err = _snd_pcm_hw_param_set_max(params, SND_PCM_HW_PARAM_RATE,
+						rate->rate_max, 0);
+		if (err < 0)
+			return err;
+	}
 	params->info &= ~(SND_PCM_INFO_MMAP | SND_PCM_INFO_MMAP_VALID);
 	return 0;
 }
@@ -186,7 +196,7 @@ static int snd_pcm_rate_hw_refine_cchange(snd_pcm_t *pcm, snd_pcm_hw_params_t *p
 		if (!snd_interval_checkempty(period_size) &&
 		    period_size->openmin && period_size->openmax &&
 		    period_size->min + 1 == period_size->max) {
-		    	if ((buffer_size->min / period_size->min) * period_size->min == buffer_size->min) {
+			if (period_size->min > 0 && (buffer_size->min / period_size->min) * period_size->min == buffer_size->min) {
 		    		snd_interval_set_value(period_size, period_size->min);
 		    	} else if ((buffer_size->max / period_size->max) * period_size->max == buffer_size->max) {
 		    		snd_interval_set_value(period_size, period_size->max);
@@ -1178,6 +1188,9 @@ static void snd_pcm_rate_dump(snd_pcm_t *pcm, snd_output_t *out)
 		snd_output_printf(out, "Rate conversion PCM (%d, sformat=%s)\n", 
 			rate->srate,
 			snd_pcm_format_name(rate->sformat));
+	if (rate->ops.dump)
+		rate->ops.dump(rate->obj, out);
+	snd_output_printf(out, "Protocol version: %x\n", rate->plugin_version);
 	if (pcm->setup) {
 		snd_output_printf(out, "Its setup is:\n");
 		snd_pcm_dump_setup(pcm, out);
@@ -1192,10 +1205,12 @@ static int snd_pcm_rate_close(snd_pcm_t *pcm)
 
 	if (rate->ops.close)
 		rate->ops.close(rate->obj);
+	if (rate->open_func)
+		snd_dlobj_cache_put(rate->open_func);
 	return snd_pcm_generic_close(pcm);
 }
 
-static snd_pcm_fast_ops_t snd_pcm_rate_fast_ops = {
+static const snd_pcm_fast_ops_t snd_pcm_rate_fast_ops = {
 	.status = snd_pcm_rate_status,
 	.state = snd_pcm_rate_state,
 	.hwsync = snd_pcm_rate_hwsync,
@@ -1219,9 +1234,10 @@ static snd_pcm_fast_ops_t snd_pcm_rate_fast_ops = {
 	.poll_descriptors_count = snd_pcm_generic_poll_descriptors_count,
 	.poll_descriptors = snd_pcm_generic_poll_descriptors,
 	.poll_revents = snd_pcm_rate_poll_revents,
+	.may_wait_for_avail_min = snd_pcm_generic_may_wait_for_avail_min,
 };
 
-static snd_pcm_ops_t snd_pcm_rate_ops = {
+static const snd_pcm_ops_t snd_pcm_rate_ops = {
 	.close = snd_pcm_rate_close,
 	.info = snd_pcm_generic_info,
 	.hw_refine = snd_pcm_rate_hw_refine,
@@ -1234,6 +1250,9 @@ static snd_pcm_ops_t snd_pcm_rate_ops = {
 	.async = snd_pcm_generic_async,
 	.mmap = snd_pcm_generic_mmap,
 	.munmap = snd_pcm_generic_munmap,
+	.query_chmaps = snd_pcm_generic_query_chmaps,
+	.get_chmap = snd_pcm_generic_get_chmap,
+	.set_chmap = snd_pcm_generic_set_chmap,
 };
 
 /**
@@ -1256,36 +1275,50 @@ static int is_builtin_plugin(const char *type)
 	return strcmp(type, "linear") == 0;
 }
 
-static const char *default_rate_plugins[] = {
+static const char *const default_rate_plugins[] = {
 	"speexrate", "linear", NULL
 };
 
-static int rate_open_func(snd_pcm_rate_t *rate, const char *type)
+static int rate_open_func(snd_pcm_rate_t *rate, const char *type, int verbose)
 {
-	char open_name[64];
+	char open_name[64], lib_name[128], *lib = NULL;
 	snd_pcm_rate_open_func_t open_func;
+	int err;
 
 	snprintf(open_name, sizeof(open_name), "_snd_pcm_rate_%s_open", type);
-	open_func = snd_dlobj_cache_lookup(open_name);
-	if (!open_func) {
-		void *h;
-		char lib_name[128], *lib = NULL;
-		if (!is_builtin_plugin(type)) {
-			snprintf(lib_name, sizeof(lib_name),
+	if (!is_builtin_plugin(type)) {
+		snprintf(lib_name, sizeof(lib_name),
 				 "%s/libasound_module_rate_%s.so", ALSA_PLUGIN_DIR, type);
-			lib = lib_name;
-		}
-		h = snd_dlopen(lib, RTLD_NOW);
-		if (!h)
-			return -ENOENT;
-		open_func = snd_dlsym(h, open_name, NULL);
-		if (!open_func) {
-			snd_dlclose(h);
-			return -ENOENT;
-		}
-		snd_dlobj_cache_add(open_name, h, open_func);
+		lib = lib_name;
 	}
-	return open_func(SND_PCM_RATE_PLUGIN_VERSION, &rate->obj, &rate->ops);
+	open_func = snd_dlobj_cache_get(lib, open_name, NULL, verbose);
+	if (!open_func)
+		return -ENOENT;
+
+	rate->open_func = open_func;
+	rate->rate_min = SND_PCM_PLUGIN_RATE_MIN;
+	rate->rate_max = SND_PCM_PLUGIN_RATE_MAX;
+	rate->plugin_version = SND_PCM_RATE_PLUGIN_VERSION;
+
+	err = open_func(SND_PCM_RATE_PLUGIN_VERSION, &rate->obj, &rate->ops);
+	if (!err) {
+		rate->plugin_version = rate->ops.version;
+		if (rate->ops.get_supported_rates)
+			rate->ops.get_supported_rates(rate->obj,
+						      &rate->rate_min,
+						      &rate->rate_max);
+		return 0;
+	}
+
+	/* try to open with the old protocol version */
+	rate->plugin_version = SND_PCM_RATE_PLUGIN_VERSION_OLD;
+	err = open_func(SND_PCM_RATE_PLUGIN_VERSION_OLD,
+			&rate->obj, &rate->ops);
+	if (err) {
+		snd_dlobj_cache_put(open_func);
+		rate->open_func = NULL;
+	}
+	return err;
 }
 #endif
 
@@ -1295,7 +1328,7 @@ static int rate_open_func(snd_pcm_rate_t *rate, const char *type)
  * \param name Name of PCM
  * \param sformat Slave format
  * \param srate Slave rate
- * \param type SRC type string
+ * \param converter SRC type string node
  * \param slave Slave PCM handle
  * \param close_slave When set, the slave PCM handle is closed with copy PCM
  * \retval zero on success otherwise a negative error code
@@ -1340,34 +1373,36 @@ int snd_pcm_rate_open(snd_pcm_t **pcmp, const char *name,
 #ifdef PIC
 	err = -ENOENT;
 	if (!converter) {
-		const char **types;
+		const char *const *types;
 		for (types = default_rate_plugins; *types; types++) {
-			err = rate_open_func(rate, *types);
+			err = rate_open_func(rate, *types, 0);
 			if (!err) {
 				type = *types;
 				break;
 			}
 		}
 	} else if (!snd_config_get_string(converter, &type))
-		err = rate_open_func(rate, type);
+		err = rate_open_func(rate, type, 1);
 	else if (snd_config_get_type(converter) == SND_CONFIG_TYPE_COMPOUND) {
 		snd_config_iterator_t i, next;
 		snd_config_for_each(i, next, converter) {
 			snd_config_t *n = snd_config_iterator_entry(i);
 			if (snd_config_get_string(n, &type) < 0)
 				break;
-			err = rate_open_func(rate, type);
+			err = rate_open_func(rate, type, 0);
 			if (!err)
 				break;
 		}
 	} else {
 		SNDERR("Invalid type for rate converter");
-		snd_pcm_close(pcm);
+		snd_pcm_free(pcm);
+		free(rate);
 		return -EINVAL;
 	}
 	if (err < 0) {
 		SNDERR("Cannot find rate converter");
-		snd_pcm_close(pcm);
+		snd_pcm_free(pcm);
+		free(rate);
 		return -ENOENT;
 	}
 #else
@@ -1375,7 +1410,8 @@ int snd_pcm_rate_open(snd_pcm_t **pcmp, const char *name,
 	open_func = SND_PCM_RATE_PLUGIN_ENTRY(linear);
 	err = open_func(SND_PCM_RATE_PLUGIN_VERSION, &rate->obj, &rate->ops);
 	if (err < 0) {
-		snd_pcm_close(pcm);
+		snd_pcm_free(pcm);
+		free(rate);
 		return err;
 	}
 #endif
@@ -1383,7 +1419,8 @@ int snd_pcm_rate_open(snd_pcm_t **pcmp, const char *name,
 	if (! rate->ops.init || ! (rate->ops.convert || rate->ops.convert_s16) ||
 	    ! rate->ops.input_frames || ! rate->ops.output_frames) {
 		SNDERR("Inproper rate plugin %s initialization", type);
-		snd_pcm_close(pcm);
+		snd_pcm_free(pcm);
+		free(rate);
 		return err;
 	}
 
