@@ -67,6 +67,7 @@ typedef struct {
 	int use_getput;
 	unsigned int src_size;
 	snd_pcm_format_t dst_sfmt;
+	unsigned int nsrcs;
 	unsigned int ndsts;
 	snd_pcm_route_ttable_dst_t *dsts;
 } snd_pcm_route_params_t;
@@ -236,14 +237,14 @@ static void snd_pcm_route_convert1_many(const snd_pcm_channel_area_t *dst_area,
 #include "plugin_ops.h"
 #undef GETS_LABELS
 #undef PUT32_LABELS
-	static void *zero_labels[3] = {
+	static void *const zero_labels[3] = {
 		&&zero_int32, &&zero_int64,
 #if SND_PCM_PLUGIN_ROUTE_FLOAT
 		&&zero_float
 #endif
 	};
 	/* sum_type att */
-	static void *add_labels[3 * 2] = {
+	static void *const add_labels[3 * 2] = {
 		&&add_int32_noatt, &&add_int32_att,
 		&&add_int64_noatt, &&add_int64_att,
 #if SND_PCM_PLUGIN_ROUTE_FLOAT
@@ -251,7 +252,7 @@ static void snd_pcm_route_convert1_many(const snd_pcm_channel_area_t *dst_area,
 #endif
 	};
 	/* sum_type att shift */
-	static void *norm_labels[3 * 2 * 4] = {
+	static void *const norm_labels[3 * 2 * 4] = {
 		0,
 		&&norm_int32_8_noatt,
 		&&norm_int32_16_noatt,
@@ -703,6 +704,47 @@ snd_pcm_route_read_areas(snd_pcm_t *pcm,
 	return size;
 }
 
+static snd_pcm_chmap_t *snd_pcm_route_get_chmap(snd_pcm_t *pcm)
+{
+	snd_pcm_route_t *route = pcm->private_data;
+	snd_pcm_chmap_t *map, *slave_map;
+	unsigned int src, dst, nsrcs;
+
+	slave_map = snd_pcm_generic_get_chmap(pcm);
+	if (!slave_map)
+		return NULL;
+	nsrcs = route->params.nsrcs;
+	map = calloc(4, nsrcs + 1);
+	if (!map) {
+		free(slave_map);
+		return NULL;
+	}
+	map->channels = nsrcs;
+	for (src = 0; src < nsrcs; src++)
+		map->pos[src] = SND_CHMAP_NA;
+	for (dst = 0; dst < route->params.ndsts; dst++) {
+		snd_pcm_route_ttable_dst_t *d = &route->params.dsts[dst];
+		for (src = 0; src < d->nsrcs; src++) {
+			unsigned int c = d->srcs[src].channel;
+			if (c < nsrcs && map->pos[c] == SND_CHMAP_NA)
+				map->pos[c] = slave_map->pos[dst];
+		}
+	}
+	free(slave_map);
+	return map;
+}
+
+static snd_pcm_chmap_query_t **snd_pcm_route_query_chmaps(snd_pcm_t *pcm)
+{
+	snd_pcm_chmap_query_t **maps;
+	snd_pcm_chmap_t *map = snd_pcm_route_get_chmap(pcm);
+	if (!map)
+		return NULL;
+	maps = _snd_pcm_make_single_query_chmaps(map);
+	free(map);
+	return maps;
+}
+
 static void snd_pcm_route_dump(snd_pcm_t *pcm, snd_output_t *out)
 {
 	snd_pcm_route_t *route = pcm->private_data;
@@ -747,7 +789,7 @@ static void snd_pcm_route_dump(snd_pcm_t *pcm, snd_output_t *out)
 	snd_pcm_dump(route->plug.gen.slave, out);
 }
 
-static snd_pcm_ops_t snd_pcm_route_ops = {
+static const snd_pcm_ops_t snd_pcm_route_ops = {
 	.close = snd_pcm_route_close,
 	.info = snd_pcm_generic_info,
 	.hw_refine = snd_pcm_route_hw_refine,
@@ -760,6 +802,9 @@ static snd_pcm_ops_t snd_pcm_route_ops = {
 	.async = snd_pcm_generic_async,
 	.mmap = snd_pcm_generic_mmap,
 	.munmap = snd_pcm_generic_munmap,
+	.query_chmaps = snd_pcm_route_query_chmaps,
+	.get_chmap = snd_pcm_route_get_chmap,
+	.set_chmap = NULL, /* NYI */
 };
 
 static int route_load_ttable(snd_pcm_route_params_t *params, snd_pcm_stream_t stream,
@@ -782,6 +827,7 @@ static int route_load_ttable(snd_pcm_route_params_t *params, snd_pcm_stream_t st
 		dmul = tt_ssize;
 	}
 	params->ndsts = dused;
+	params->nsrcs = sused;
 	dptr = calloc(dused, sizeof(*params->dsts));
 	if (!dptr)
 		return -ENOMEM;
