@@ -43,7 +43,7 @@ struct _snd_rawmidi {
 	snd_rawmidi_stream_t stream;
 	int mode;
 	int poll_fd;
-	snd_rawmidi_ops_t *ops;
+	const snd_rawmidi_ops_t *ops;
 	void *private_data;
 	size_t buffer_size;
 	size_t avail_min;
@@ -53,3 +53,9 @@ struct _snd_rawmidi {
 int snd_rawmidi_hw_open(snd_rawmidi_t **input, snd_rawmidi_t **output,
 			const char *name, int card, int device, int subdevice,
 			int mode);
+
+int snd_rawmidi_virtual_open(snd_rawmidi_t **inputp, snd_rawmidi_t **outputp,
+			     const char *name, snd_seq_t *seq_handle, int port,
+			     int merge, int mode);
+
+int snd_rawmidi_conf_generic_id(const char *id);
